#include "meter.h"
#include "serial.h"

#define REQUEST_INFORMATION 'D'
#define MAX_TRIES           10  /* 5 seems to work, x2 for safety */

#define COM_PARAMS  (_COM_CHR7 | _COM_STOP2 | _COM_NOPARITY | _COM_1200)
#define LINE_PARAMS DTR_ON

int     OpenMeter(unsigned PortNumber) {
    return SSetup(PortNumber-1,COM_PARAMS,LINE_PARAMS);
}

int     CloseMeter(void) {
    return SClose();
}

int     GetMeterReading(char *Reading) {
    int     Result;
    int     AnyReceived;
    int     NumCopied;
    int     Tries;
    int     Ch;

    Result = 0;

    if (!SSendChar(REQUEST_INFORMATION))
        strcpy(Reading,"Failed to send.");
    else {
        AnyReceived = 0;
        for (Tries = 0; AnyReceived == 0 && Tries < MAX_TRIES; Tries++)
            if ((Ch = SRcvChar()) != -1)
                AnyReceived = 1;
        if (AnyReceived == 0)
            strcpy(Reading,"Failed to receive.");
        else {
            NumCopied = 0;
            do {
                if (Ch != '\r' && NumCopied < METER_READING_SIZE-1) {
                    *Reading++ = Ch;
                    NumCopied++;
                }
            } while ((Ch = SRcvChar()) != -1);
            *Reading = '\0';
            Result = 1;
        }
    }
    return Result;
}

#ifdef  FOR_TEST

/***************************************************************/

#include <time.h>
#include <stdio.h>

#define CONTROL(x)  ((x)-0x40)
#define ESC_KEY     CONTROL('[')


int     main(int Argc,char *Argv[]) {
    char    Reading[METER_READING_SIZE];
    time_t  Time;
    char    TimeBuf[26];
    int     PortNumber;

    PortNumber = 1;
    if (Argc > 1)
        PortNumber = atoi(Argv[1]);

    fprintf(stderr,
        "METER -- Getting Radio Shack Digital Meter Readings from COM%d\n",
        PortNumber);
    fprintf(stderr,"\n");
    fprintf(stderr,"Press ESC to exit.\n");
    fprintf(stderr,"\n");

    OpenMeter(PortNumber);

    for (;;) {
        if (kbhit() && getch() == ESC_KEY)
            break;

        Time = time(NULL);
        strcpy(TimeBuf,ctime(&Time));
        TimeBuf[24] = '\0';

        if (GetMeterReading(Reading))
            printf("%s %s\n",TimeBuf,Reading);
        else
            printf("%s %s (Oops!)\n",TimeBuf,Reading);
    }
    CloseMeter();
    return 0;
}
#endif
